<?php
/* --------------------------------------------------------------
   HubCheckoutHelper.inc.php 2017-07-17
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2017 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

use \HubPublic\Collections\CartItemCollection;
use \HubPublic\Exceptions\CurlRequestException;
use \HubPublic\Http\CurlRequest;
use \HubPublic\Serializers\CartContentSerializer;
use \HubPublic\Serializers\CustomerInformationSerializer;
use \HubPublic\Serializers\HubClientInformationSerializer;
use \HubPublic\Serializers\OrderContentSerializer;
use \HubPublic\Serializers\ClientSessionInformationSerializer;
use \HubPublic\ValueObjects\Builder\CustomerInformation as CustomerInformationBuilder;
use \HubPublic\ValueObjects\CartContent;
use \HubPublic\ValueObjects\CartItem;
use \HubPublic\ValueObjects\HubClientInformation;
use \HubPublic\ValueObjects\HubClientKey;
use \HubPublic\ValueObjects\HubSessionKey;
use \HubPublic\ValueObjects\ClientSessionInformation;

/**
 * Class HubCheckoutHelper
 */
class HubCheckoutHelper
{
	/**
	 * Creates a HubTransactionsApiClient instance.
	 *
	 * @param \HubPublic\ValueObjects\HubSessionKey $sessionKey Hub session key.
	 *
	 * @return \HubTransactionsApiClient Created instance.
	 */
	public function createHubTransactionsApiClient(HubSessionKey $sessionKey)
	{
		// cURL request.
		$request = new CurlRequest();
		
		// Cart content serializer.
		$cartContentSerializer = new CartContentSerializer();
		
		// Customer information serializer.
		$customerInformationSerializer = new CustomerInformationSerializer();
		
		// Hub client information serializer.
		$hubClientInformationSerializer = new HubClientInformationSerializer();
		
		// Session information serializer.
		$clientSessionInformationSerializer = new ClientSessionInformationSerializer();
		
		// Order content serializer.
		$orderContentSerializer = new OrderContentSerializer($customerInformationSerializer);
		
		// Shop logger instance. 
		$logControl = LogControl::get_instance();
		
		// Hub settings instance.
		$hubSettings = MainFactory::create('HubSettings', gm_get_conf('GAMBIO_HUB_CURL_TIMEOUT'));
		
		return MainFactory::create('HubTransactionsApiClient', MODULE_PAYMENT_GAMBIO_HUB_URL, $sessionKey, $request,
		                           $cartContentSerializer, $customerInformationSerializer,
		                           $hubClientInformationSerializer, $clientSessionInformationSerializer,
		                           $orderContentSerializer, $logControl, $hubSettings);
	}
	
	
	/**
	 * Creates a HubSessionsApiClient instance.
	 *
	 * @param string        $shopUrl Shop url with trailing slash.
	 * @param \LanguageCode $languageCode
	 *
	 * @return bool|\HubSessionsApiClient
	 */
	public function startSession($shopUrl, LanguageCode $languageCode)
	{
		if(!is_string($shopUrl))
		{
			throw new InvalidArgumentException('Shop url argument is not a string: ' . gettype($shopUrl));
		}
		
		// start a new gambio hub session
		$coo_hub_service_factory          = MainFactory::create('HubServiceFactory');
		$coo_hub_session_key_service      = $coo_hub_service_factory->createHubSessionKeyService();
		$coo_hub_client_key_configuration = MainFactory::create('HubClientKeyConfiguration');
		$coo_curl_request                 = new \HubPublic\Http\CurlRequest();
		$coo_log_control                  = LogControl::get_instance();
		$hubSettings                      = MainFactory::create('HubSettings', gm_get_conf('GAMBIO_HUB_CURL_TIMEOUT'));
		$coo_hub_sessions_api_client      = MainFactory::create('HubSessionsApiClient', MODULE_PAYMENT_GAMBIO_HUB_URL,
		                                                        $coo_hub_session_key_service,
		                                                        $coo_hub_client_key_configuration, $coo_curl_request,
		                                                        $coo_log_control, $hubSettings);
		
		$authHash = AuthHashCreator::create();
		
		try
		{
			$coo_hub_sessions_api_client->startSession($authHash, $shopUrl, $languageCode);
		}
		catch(UnexpectedValueException $e)
		{
			AuthHashCreator::invalidate($authHash);
		}
		catch(CurlRequestException $e)
		{
			AuthHashCreator::invalidate($authHash);
		}
	}
	
	
	/**
	 * Returns a CartContent instance based on the global 'order' information.
	 *
	 * @param order $order Contains the order information.
	 *
	 * @return \HubPublic\ValueObjects\CartContent
	 */
	public function getCartContent(order $order)
	{
		$cartItemCollection = new CartItemCollection();
		
		foreach($order->products as $product)
		{
			$productEan = null;
			$attributes = [];
			
			// Parse Product Properties
			if(strpos($product['id'], 'x') !== false)
			{
				$combinationId = array_pop(explode('x', $product['id']));
				
				$sql         = '
					SELECT * 
					FROM `products_properties_combis`
					WHERE `products_properties_combis_id` = ' . (int)$combinationId . '
				';
				$query       = xtc_db_query($sql);
				$combination = xtc_db_fetch_array($query);
				
				$productEan = $combination['combi_ean'];
				
				$sql = '
					SELECT `properties_description`.`properties_name`, `properties_description`.`properties_name`
					FROM `products_properties_combis`
					INNER JOIN `products_properties_combis_values`
						ON `products_properties_combis_values`.`products_properties_combis_id` = `products_properties_combis`.`products_properties_combis_id`
					INNER JOIN `properties_values`
						ON `properties_values`.`properties_values_id` = `products_properties_combis_values`.`properties_values_id`
					INNER JOIN `properties_description`
						ON `properties_description`.`properties_id` = `properties_values`.`properties_id`
					WHERE `products_properties_combis`.`products_properties_combis_id` = ' . $combinationId . '
					AND `properties_description`.`language_id` = ' . $_SESSION['languages_id'] . '
				';
				
				$query             = xtc_db_query($sql);
				$combinationValues = [];
				
				while($row = xtc_db_fetch_array($query))
				{
					$combinationValues[] = $row;
				}
				
				foreach($combinationValues as $combinationValue)
				{
					$attributes[$combinationValue['properties_name']] = $combinationValue['value_model'];
				}
			}
			
			// Parse Product Attributes
			if(strpos($product['id'], '{') !== false)
			{
				$exploded = explode('{', $product['id']);
				
				$sql        = 'SELECT `products_ean` FROM `products` WHERE `products_id` = ' . (int)$exploded[0];
				$query      = xtc_db_query($sql);
				$productEan = xtc_db_fetch_array($query)['products_ean'];
				
				foreach($exploded as $segment)
				{
					if(strpos($segment, '}') === false)
					{
						continue;
					}
					
					$productOptionsIds = explode('}', $segment);
					
					$sql = ' 
						SELECT `products_options`.`products_options_name`, `products_options_values`.`products_options_values_name`, `products_attributes`.`gm_ean`
						FROM `products` 
						INNER JOIN `products_attributes`
							ON `products_attributes`.`products_id` = `products`.`products_id`
						INNER JOIN `products_options`
							ON `products_options`.`products_options_id` = `products_attributes`.options_id
						INNER JOIN `products_options_values_to_products_options`
							ON `products_options_values_to_products_options`.`products_options_id` = `products_options`.`products_options_id`
						INNER JOIN `products_options_values`
							ON `products_options_values`.`products_options_values_id` = `products_options_values_to_products_options`.`products_options_values_id`
						WHERE `products_options`.`products_options_id` = ' . (int)$productOptionsIds[0] . '
						AND `products_options_values`.`products_options_values_id` = ' . (int)$productOptionsIds[1] . '
						AND `products_options_values`.`language_id` = ' . (int)$_SESSION['languages_id'] . '
						GROUP BY `products_options`.`products_options_id`
					';
					
					$query          = xtc_db_query($sql);
					$productOptions = [];
					
					while($row = xtc_db_fetch_array($query))
					{
						$productOptions[] = $row;
					}
					
					foreach($productOptions as $productOption)
					{
						if(!empty($productOption['gm_ean'])) // Replace the product EAN number if there was one in attributes.
						{
							$productEan = $productOption['gm_ean'];
						}
						
						$attributes[$productOption['products_options_name']] = $productOption['products_options_values_name'];
					}
				}
			}
			
			// Prepare final cart item attributes array.
			$cartItem = new CartItem($productEan, $product['name'], $product['final_price'], $product['qty'],
			                         $attributes);
			$cartItemCollection->add($cartItem);
		}
		
		$globalsOrder     = clone $GLOBALS['order'];
		$GLOBALS['order'] = new order();
		$orderTotal       = new order_total();
		$orderTotalArray  = $orderTotal->process();
		$GLOBALS['order'] = $globalsOrder;
		$orderTotalPrice  = end($orderTotalArray)['value'];
		
		foreach($orderTotalArray as $module)
		{
			$GLOBALS[$module['code']]->output = array();
		}
		
		$cartContent = new CartContent($cartItemCollection, $orderTotalPrice);
		
		return $cartContent;
	}
	
	
	/**
	 * Returns a CustomerInformation instance based on the global 'order' information.
	 *
	 * @param order $order Contains the order information.
	 *
	 * @return \HubPublic\ValueObjects\CustomerInformation
	 */
	public function getCustomerInformation(order $order)
	{
		$customer = $this->getCustomer($_SESSION['customer_id']);
		
		$customerInformationBuilder = new CustomerInformationBuilder();
		$customerInformationBuilder->setCustomerNumber((string)$order->customer['csID'])
		                           ->setCustomerFirstName((string)$order->customer['firstname'])
		                           ->setCustomerLastName((string)$order->customer['lastname'])
		                           ->setCustomerGender((string)$order->customer['gender'])
		                           ->setCustomerCompany((string)$order->customer['company'])
		                           ->setCustomerAddress1((string)$order->customer['street_address'])
		                           ->setCustomerAddress2((string)$order->customer['additional_address_info'])
		                           ->setCustomerPostalCode((string)$order->customer['postcode'])
		                           ->setCustomerCity((string)$order->customer['city'])
		                           ->setCustomerState((string)$order->customer['state'])
		                           ->setCustomerCountry((string)$order->customer['country']['title'])
		                           ->setCustomerCountryCode((string)$order->customer['country']['iso_code_2'])
		                           ->setCustomerDateOfBirth((string)$customer['customers_dob'])
		                           ->setCustomerPhone((string)$order->customer['telephone'])
		                           ->setCustomerFax((string)$customer['customers_fax'])
		                           ->setCustomerEmail((string)$customer['customers_email_address'])
		                           ->setBillingFirstName((string)$order->billing['firstname'])
		                           ->setBillingLastName((string)$order->billing['lastname'])
		                           ->setBillingGender((string)$order->billing['gender'])
		                           ->setBillingCompany((string)$order->billing['company'])
		                           ->setBillingAddress1((string)$order->billing['street_address'])
		                           ->setBillingAddress2((string)$order->billing['additional_address_info'])
		                           ->setBillingPostalCode((string)$order->billing['postcode'])
		                           ->setBillingCity((string)$order->billing['city'])
		                           ->setBillingState((string)$order->billing['state'])
		                           ->setBillingCountry((string)$order->billing['country']['title'])
		                           ->setBillingCountryCode((string)$order->billing['country']['iso_code_2'])
		                           ->setShippingFirstName((string)$order->delivery['firstname'])
		                           ->setShippingLastName((string)$order->delivery['lastname'])
		                           ->setShippingGender((string)$order->delivery['gender'])
		                           ->setShippingCompany((string)$order->delivery['company'])
		                           ->setShippingAddress1((string)$order->delivery['street_address'])
		                           ->setShippingAddress2((string)$order->delivery['additional_address_info'])
		                           ->setShippingPostalCode((string)$order->delivery['postcode'])
		                           ->setShippingCity((string)$order->delivery['city'])
		                           ->setShippingState((string)$order->delivery['state'])
		                           ->setShippingCountry((string)$order->delivery['country']['title'])
		                           ->setShippingCountryCode((string)$order->delivery['country']['iso_code_2']);
		
		return $customerInformationBuilder->build();
	}
	
	
	/**
	 * Returns a HubClientInformation instance
	 *
	 * @return \HubPublic\ValueObjects\HubClientInformation
	 */
	public function getHubClientInformation()
	{
		$hubClientKeyConfiguration = MainFactory::create('HubClientKeyConfiguration');
		$hubClientKey              = new HubClientKey($hubClientKeyConfiguration->get());
		$hubClientInformation      = new HubClientInformation($hubClientKey,
		                                                      ltrim(gm_get_conf('INSTALLED_VERSION'), 'v'),
		                                                      HTTP_SERVER . DIR_WS_CATALOG);
		
		return $hubClientInformation;
	}
	
	
	/**
	 * Returns a ClientSessionInformation instance.
	 *
	 * @param order $order Contains the order information.
	 *
	 * @return \HubPublic\ValueObjects\ClientSessionInformation
	 */
	public function getClientSessionInformation(order $order)
	{
		$clientSessionInformation = new ClientSessionInformation(new HubSessionKey($_SESSION['gambio_hub_session_key']),
		                                                         $_SESSION['language_code'], $order->info['currency'],
		                                                         $_SERVER['REMOTE_ADDR'], $_SERVER['HTTP_USER_AGENT']);
		
		return $clientSessionInformation;
	}
	
	
	/**
	 * Get customer information from database.
	 *
	 * @param int $customerId The customer's row ID.
	 *
	 * @return array Returns an array with the customer's data.
	 */
	public function getCustomer($customerId)
	{
		$sql   = 'SELECT * FROM `customers` WHERE `customers_id` = ' . (int)$customerId;
		$query = xtc_db_query($sql);
		
		return xtc_db_fetch_array($query);
	}
}